# Generated by Django 5.0 on 2024-01-05 15:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0008_dataservice"),
    ]

    operations = [
        migrations.AlterField(
            model_name="agent",
            name="type",
            field=models.CharField(blank=True, max_length=20),
        ),
    ]
