# Generated by Django 5.0 on 2023-12-20 23:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0003_catalog_homepage"),
    ]

    operations = [
        migrations.AlterField(
            model_name="catalog",
            name="themes",
            field=models.ManyToManyField(blank=True, to="dcat.datatheme"),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="description",
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="themes",
            field=models.ManyToManyField(blank=True, to="dcat.datatheme"),
        ),
    ]
