# Generated by Django 5.0 on 2023-12-21 17:02

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0005_keyword_dataset_keywords"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datatheme",
            name="code",
            field=models.CharField(max_length=255, unique=True),
        ),
    ]
